/** \file board.cpp
 * Implements class Board
 * \author Robert W Martin
 * \date 03-19-2007
 */

/**
 * Include Files
 */
#include "board.h"
#include "piece.h"
#include <iostream>
/**
 * Namespaces
 */
using namespace Interface;

/**
 * Namespace Game
 */
namespace Game {

	/**
	 * Implementation for class Board
	 */
	Board::Board( void ) {
		// do nothing
	}
	Board::~Board( void ) {
		// do nothing
	}
	void Board::Initialize( void ) {

		// Initialize the Game State...
		m_Turn = WHITE;
		m_Status = GS_NORMAL;
		m_pSelected = NULL;

		// Create the Pawns...
		for( int i = 0; i< Spaces; ++i ) {
			m_Pieces.push_back( new Pawn( this, WHITE, i, PAWN_ROW[WHITE] ) );
			m_Pieces.push_back( new Pawn( this, BLACK, i, PAWN_ROW[BLACK] ) );
		}

		// Create the Bishops...
		m_Pieces.push_back( new Bishop( this, WHITE, 2, HOME_ROW[WHITE] ) );
		m_Pieces.push_back( new Bishop( this, WHITE, 5, HOME_ROW[WHITE] ) );
		m_Pieces.push_back( new Bishop( this, BLACK, 2, HOME_ROW[BLACK] ) );
		m_Pieces.push_back( new Bishop( this, BLACK, 5, HOME_ROW[BLACK] ) );

		// Create the Rooks...
		m_Pieces.push_back( new Rook( this, WHITE, 0, HOME_ROW[WHITE] ) );
		m_Pieces.push_back( new Rook( this, WHITE, 7, HOME_ROW[WHITE] ) );
		m_Pieces.push_back( new Rook( this, BLACK, 0, HOME_ROW[BLACK] ) );
		m_Pieces.push_back( new Rook( this, BLACK, 7, HOME_ROW[BLACK] ) );

		// Create the Knights...
		m_Pieces.push_back( new Knight( this, WHITE, 1, HOME_ROW[WHITE] ) );
		m_Pieces.push_back( new Knight( this, WHITE, 6, HOME_ROW[WHITE] ) );
		m_Pieces.push_back( new Knight( this, BLACK, 1, HOME_ROW[BLACK] ) );
		m_Pieces.push_back( new Knight( this, BLACK, 6, HOME_ROW[BLACK] ) );

		// Create the Queens...
		m_Pieces.push_back( new Queen( this, WHITE, 3, HOME_ROW[WHITE] ) );
		m_Pieces.push_back( new Queen( this, BLACK, 3, HOME_ROW[BLACK] ) );

		// Create the Kings...
		m_Pieces.push_back( new King( this, WHITE, 4, HOME_ROW[WHITE] ) );
		m_Pieces.push_back( new King( this, BLACK, 4, HOME_ROW[BLACK] ) );

		// Generate possible moves
		PieceIterator iter = m_Pieces.begin();
		while( iter != m_Pieces.end() ) {
			(*iter)->RefreshMoves();
			++iter;
		}

	}
	void Board::Shutdown( void ) {

		PieceIterator iter = m_Pieces.begin();
		while( iter != m_Pieces.end() ) {
			delete (*iter);
			(*iter) = NULL;
			++iter;
		}
		m_Pieces.clear();

	}
	void Board::Draw( void ) {

		OpenGL *pGL = OpenGL::GetSingleton();

		// Board Colors...
		Color cDark( 0.5f, 0.5f, 0.5f ), cLight( 0.75f, 0.75f, 0.75f );
		Color cBack( 0.5f, 0.25f, 0.0f );
		Color cSplash( 0.5f, 0.5f, 0.75f );

		// Render the background
		cSplash.Activate();
		pGL->Begin( GL_QUADS );

		pGL->Vertex( ScreenL, ScreenT, -1 );
		pGL->Vertex( ScreenR, ScreenT, -1 );
		pGL->Vertex( ScreenR, ScreenB, -1 );
		pGL->Vertex( ScreenL, ScreenB, -1 );

		pGL->End();

		// Set Up our Matrix...
		pGL->Push();
		pGL->Identity();
		pGL->Translate( BoardStartX - SpaceSize / 2.0f, BoardStartY - SpaceSize / 2.0f, 0.0f );

		// Render the board backing...
		cBack.Activate();
		pGL->Begin( GL_QUADS );

		pGL->Vertex( -Border,						-Border,					-1 );
		pGL->Vertex( BoardSize * 2.0f + Border, 	-Border,					-1 );
		pGL->Vertex( BoardSize * 2.0f + Border, 	BoardSize * 2.0f+ Border,	-1 );
		pGL->Vertex( -Border,						BoardSize * 2.0f+ Border,	-1 );

		pGL->End();

		// Restore the Matrix...
		pGL->Pop();

		// Render each Board Space...
		BoardSpace bs = { 0, 0 };
		for( bs.iX = 0; bs.iX < Spaces; ++bs.iX ) {
			for( bs.iY = 0; bs.iY < Spaces; ++bs.iY ) {
				DrawSpace( bs, ( ( ( bs.iX + bs.iY ) % 2 ) == 1 ) ? cDark : cLight );
			}
		}

		// Render the Highlight[s] for the Selected Piece...
		if( m_pSelected ) {
			pGL->Blend( true );
			DrawSpace( m_pSelected->GetPosition(), Color( 0.0f, 0.0f, 1.0f, 0.75f ) );
			SpaceList moves = m_pSelected->GetMoves();
			SpaceIterator iter = moves.begin();
			while( iter != moves.end() ) {
				if( (*iter).bLegal ) DrawSpace( (*iter), Color( 0.0f, 1.0f, 1.0f, 0.5f ) );
				++iter;
			}
			pGL->Blend( false );
		}

		// Render each Board Piece...
		PieceIterator iter = m_Pieces.begin();
		while( iter != m_Pieces.end() ) {
			(*iter)->Draw();
			++iter;
		}

		// Draw the Current Turn
		float tX = (float)pGL->Width() * 0.7f;
		float tY = (float)pGL->Height() * 0.9f;
		if( m_Turn == WHITE ) pGL->String( "White to Move", (int)tX, (int)tY );
		if( m_Turn == BLACK ) pGL->String( "Black to Move", (int)tX, (int)tY );

		// Draw the Current State
		tY = (float)pGL->Height() * 0.8f;
		if( m_Status == GS_CHECK[WHITE] ) pGL->String( "White King in Check", (int)tX, (int)tY );
		if( m_Status == GS_CHECK[BLACK] ) pGL->String( "Black King in Check", (int)tX, (int)tY );
		if( m_Status == GS_CMATE[WHITE] ) pGL->String( "Checkmate! White Wins", (int)tX, (int)tY );
		if( m_Status == GS_CMATE[BLACK] ) pGL->String( "Checkmate! Black Wins", (int)tX, (int)tY );

	}
	void Board::DrawSpace( BoardSpace bs, Color c ) {

		// sanity checks
		if( bs.iX < 0 || bs.iX >= Spaces || bs.iY < 0 || bs.iY >= Spaces ) return;

		OpenGL *pGL = OpenGL::GetSingleton();

		// Activate the Color
		c.Activate();

		// Set up our Matrix...
		pGL->Push();
		pGL->Identity();
		pGL->Translate( BoardStartX + SpaceSize * bs.iX, BoardStartY + SpaceSize * bs.iY, 0.0f );

		// Render the Board Space...
		pGL->Begin( GL_QUADS );
		pGL->Vertex( -SpaceSize / 2.0f, -SpaceSize / 2.0f, -1 );
		pGL->Vertex(  SpaceSize / 2.0f, -SpaceSize / 2.0f, -1 );
		pGL->Vertex(  SpaceSize / 2.0f,  SpaceSize / 2.0f, -1 );
		pGL->Vertex( -SpaceSize / 2.0f,  SpaceSize / 2.0f, -1 );
		pGL->End();

		// Restore the Matrix...
		pGL->Pop();

	}
	void Board::SelectPiece( BoardSpace bs ) {

		// sanity check...
		if( bs.iX == -1 || bs.iY == -1 ) m_pSelected = NULL;

		// check each piece...
		PieceIterator iter = m_Pieces.begin();
		while( iter != m_Pieces.end() ) {

			BoardSpace pos = (*iter)->GetPosition();
			if( pos.iX == bs.iX && pos.iY == bs.iY && (*iter)->Owner() == m_Turn && (*iter)->Alive() ) {
				m_pSelected = (*iter);
				return;
			}

			++iter;
		}

	}
	void Board::MovePiece( BoardSpace move ) {

		// sanity check...
		if( m_pSelected == NULL ) return;

		// Make sure move is legal...
		SpaceList moves = m_pSelected->GetMoves();
		SpaceIterator iter = moves.begin();
		while( iter != moves.end() ) {

			BoardSpace legal = (*iter);
			if( move.iX == legal.iX && move.iY == legal.iY && legal.bLegal ) {

				// Execute any Special Move...
				ExecuteSpecial( m_pSelected, legal );

				// Make any Relevant Captures...
				LPPIECE pOccupant = GetPiece( move );
				if( pOccupant ) {
					pOccupant->Capture();
				}

				// Move the Piece..
				m_pSelected->Move( move );

				// Promote the Piece, if needed...
				if( m_pSelected->CanBePromoted() ) {

					m_pSelected->Capture();
					m_Pieces.push_back( new Queen( this, m_pSelected->Owner(), move.iX, move.iY ) );

				}

				// Remove reference to selected piece...
				m_pSelected = NULL;

				// Advance the Turn...
				m_Turn = (m_Turn == WHITE) ? BLACK : WHITE;

				// Generate the List of Moves for the pieces...
				PieceIterator pieces = m_Pieces.begin();
				while( pieces != m_Pieces.end() ) {
					(*pieces)->RefreshMoves();
					++pieces;
				}

				// Scan the State of the Game
				ScanStatus();

				// abort loop
				return;

			}

			++iter;
		}

	}
	LPPIECE Board::GetPiece( BoardSpace pos ) {

		PieceIterator iter = m_Pieces.begin();
		while( iter != m_Pieces.end() ) {

			BoardSpace piece = (*iter)->GetPosition();
			if( piece.iX == pos.iX && piece.iY == pos.iY && (*iter)->Alive() ) return (*iter);

			++iter;
		}

		return NULL;

	}
	LPPIECE Board::GetKing( Player pl ) {

		PieceIterator iter = m_Pieces.begin();
		while( iter != m_Pieces.end() ) {

			if( (*iter)->Owner() == pl && (*iter)->VulnerableToCheck() ) {
				return (*iter);
			}

			++iter;
		}
		return NULL;

	}
	bool Board::Threatened( BoardSpace pos, Player p ) {

		PieceIterator pieces = m_Pieces.begin();
		while( pieces != m_Pieces.end() ) {

			// Ignore pieces from other player
			if( (*pieces)->Owner() != p || !(*pieces)->Alive() ) {
				++pieces;
				continue;
			}

			// See if this piece threatens us...
			SpaceList spaces = (*pieces)->GetMoves();
			SpaceIterator iter = spaces.begin();
			while( iter != spaces.end() ) {

				BoardSpace spot = (*iter);
				if( pos.iX == spot.iX && pos.iY == spot.iY && spot.bLegal && spot.bThreat ) return true;

				++iter;
			}

			++pieces;
		}

		return false;

	}
	void Board::ScanStatus( void ) {

		// Quick-Escape...
		if( m_Status == GS_CMATE[BLACK] || m_Status == GS_CMATE[WHITE] ) return;

		// Scan for Check...
		for( Player p = 0; p < NUM_PLAYERS; ++p ) {

			LPPIECE pKing = GetKing( p );
			if( pKing && Threatened( pKing->GetPosition(), OPPOSITE[pKing->Owner()] ) ) {
				m_Status = GS_CHECK[p];
				break;
			}

		}

		// Scan for CheckMate...
		if( m_Status == GS_CHECK[BLACK] || m_Status == GS_CHECK[WHITE] ) {

			// Find the player who is in check
			Player pl = (m_Status == GS_CHECK[BLACK]) ? BLACK : WHITE;

			// See if he has any available moves...
			int iMoves = 0;
			PieceIterator iter = m_Pieces.begin();
			while( iMoves == 0 && iter != m_Pieces.end() ) {

				if( (*iter)->Owner() == pl ) {

					SpaceList vSpaces = (*iter)->GetMoves();
					SpaceIterator space = vSpaces.begin();
					while( space != vSpaces.end() ) {
						if( (*space).bLegal ) {
							iMoves++;
							break;
						}
						++space;
					}

				}
				++iter;

			}

			if( iMoves == 0 ) {
				m_Status = GS_CMATE[OPPOSITE[pl]];
				m_Turn = NUM_PLAYERS;
			}
			return;

		}

		// Set Game State to Normal
		m_Status = GS_NORMAL;

	}
	bool Board::VerifyMove( LPPIECE pPiece, BoardSpace move ) {

		// Store the old state...
		State oldState = m_Status;
		Player oldTurn = m_Turn;
		bool bAnswer = true;

		// Take a Snapshot of the pieces...
		PieceIterator iter = m_Pieces.begin();
		while( iter != m_Pieces.end() ) {
			(*iter)->Snapshot();
			++iter;
		}

		// Make any Relevant Captures...
		LPPIECE pOccupant = GetPiece( move );
		if( pOccupant ) {
			pOccupant->Capture();
		}

		// Move the Piece..
		pPiece->m_Pos.iX = move.iX;
		pPiece->m_Pos.iY = move.iY;

		// Check to see if any piece is invalid...
		iter = m_Pieces.begin();
		while( bAnswer && iter != m_Pieces.end() ) {

			if( (*iter)->Owner() == pPiece->Owner() ) {
				++iter;
				continue;
			}

			SpaceList vSpaces;
			(*iter)->GenerateMoves( &vSpaces );
			SpaceIterator space = vSpaces.begin();
			while( space != vSpaces.end() ) {

				BoardSpace king = GetKing( pPiece->Owner() )->GetPosition();
				if( king.iX == (*space).iX && king.iY == (*space).iY && (*space).bLegal ) {
					bAnswer = false;
					break;
				}

				++space;
			}

			++iter;
		}

		// Restore the Snapshot...
		iter = m_Pieces.begin();
		while( iter != m_Pieces.end() ) {
			(*iter)->Revert();
			++iter;
		}

		// Restore the old state...
		m_Status = oldState;
		m_Turn = oldTurn;
		return bAnswer;

	}
	void Board::ExecuteSpecial( LPPIECE pPiece, BoardSpace move ) {

		// En Passant...
		if( move.iSpecial == SM_EN_PASSANT ) {

			BoardSpace cap = { move.iX, move.iY + BACKWARD[pPiece->Owner()], true, SM_NONE };

			LPPIECE pCapture = GetPiece( cap );
			if( pCapture ) pCapture->Capture();

		}

		// Any Castle...
		if( move.iSpecial == SM_SHORT_CASTLE || move.iSpecial == SM_LONG_CASTLE ) {

			BoardSpace from = { ((move.iSpecial == SM_SHORT_CASTLE) ? 7 : 0), move.iY, true, SM_NONE };
			BoardSpace to = { move.iX + ((move.iSpecial == SM_SHORT_CASTLE) ? -1 : 1), move.iY, true, SM_NONE };

			LPPIECE target = GetPiece( from );
			if( target ) target->Move( to );

		}

	}

}// namespace Game
