/** \file board.h
 * Defines class Board
 * \author Robert W Martin
 * \date 03-19-2007
 */

#ifndef _BOARD_H_
#define _BOARD_H_

/**
 * Include Files
 */
#include "interface.h"
#include "root.h"

/**
 * Namespace Game
 */
namespace Game {

	/** \class Board
	 * Represents a chess Board
	 */
	class Board {

	private:
		PieceList			m_Pieces;
		Player				m_Turn;
		State				m_Status;
		LPPIECE				m_pSelected;

	public:
		Board( void );
		~Board( void );
		/** \fn void Initialize( void )
		 * Initialize the Chess Board
		 * \return void
		 */
		void Initialize( void );
		/** \fn void Shutdown( void )
		 * Shuts down the Chess Board
		 * \return void
		 */
		void Shutdown( void );
		/** \fn void Draw( void )
		 * Draws the Board and the Pieces
		 * \return void
		 */
		void Draw( void );
		/** \fn void DrawSpace( BoardSpace bs, Interface::Color c, float fZ )
		 * Draws a Single Space of the Chess Board
		 * \param bs Board Space to draw
		 * \param c Color to draw the space with
		 * \return void
		 */
		void DrawSpace( BoardSpace bs, Interface::Color c );
		/** \fn void SelectPiece( BoardSpace bs )
		 * Selects the piece at the location if it belongs to the player whose turn it is
		 * \param bs Board Space where the selection was made
		 * \return void
		 */
		void SelectPiece( BoardSpace bs );
		/** \fn void MovePiece( BoardSpace move )
		 * Moves the Selected Piece to the given position, if it is a legal move
		 * \param move Move to try and attempt
		 * \return void
		 */
		void MovePiece( BoardSpace move );
		/** \fn LPPIECE GetPiece( BoardSpace pos )
		 * Gets the Piece occupying the given space
		 * \param pos Space to grab piece from
		 * \return Pointer to piece at given position, NULL if it is an empty space
		 */
		LPPIECE GetPiece( BoardSpace pos );
		/** \fn LPPIECE GetKing( Player pl )
		 * Gets the King belonging to a specific player
		 * \param pl Player whose king we're looking for
		 * \return Pointer to the player's king
		 */
		LPPIECE GetKing( Player pl );
		/** \fn bool Threatened( BoardSpace pos, Player p )
		 * Determines if the given position is threatened by the given player
		 * \param pos Position to check
		 * \param p Player that does the threatening
		 * \return true if the position is threatened, false otherwise
		 */
		bool Threatened( BoardSpace pos, Player p );
		/** \fn void ScanStatus( void )
		 * Scans the Status of the game after each move is made
		 * \return void
		 */
		void ScanStatus( void );
		/** \fn bool VerifyMove( LPPIECE pPiece, BoardSpace move )
		 * Verifys that a given move is legal
		 * \param pPiece piece that is making the move
		 * \param move Move to check
		 * \return true if the move is legal, false otherwise
		 */
		bool VerifyMove( LPPIECE pPiece, BoardSpace move );
		/** \fn void ExecuteSpecial( LPPIECE pPiece, BoardSpace move )
		 * Executes a Special move (En Passant, Castling, etc)
		 * \param pPiece The piece making the move
		 * \param move The move to make
		 * \return void
		 */
		void ExecuteSpecial( LPPIECE pPiece, BoardSpace move );

	};// class Board

}// namespace Game

#endif
