/** \file piece.cpp
 * Implements class Piece and its Subclasses
 * \author Robert W Martin
 * \date 03-19-2007
 */

/**
 * Include Files
 */
#include "piece.h"
#include "board.h"

/**
 * Namespaces
 */
using namespace Interface;

/**
 * Namespace Game
 */
namespace Game {

	/**
	 * Implementation for class Piece
	 */
	Piece::Piece( Board *pBoard, Player owner, int x, int y ) {
		m_pBoard = pBoard;
		m_bAlive = true;
		m_Owner = owner;
		m_Pos.iX = x;
		m_Pos.iY = y;
		m_iMoves = 0;
	}
	Piece::~Piece( void ) {
		// do nothing
	}
	SpaceList Piece::GetMoves( void ) {
		return m_Moves;
	}
	bool Piece::Move( BoardSpace move ) {

		if( !m_bAlive ) return false;

		m_Pos = move;
		RefreshMoves();
		m_iMoves++;
		return true;

	}
	void Piece::Capture( void ) {
		m_bAlive = false;
		m_Pos.iX = -1;
		m_Pos.iY = -1;
	}
	Player Piece::Owner( void ) {
		return m_Owner;
	}
	BoardSpace Piece::GetPosition( void ) {
		return m_Pos;
	}
	bool Piece::Alive( void ) {
		return m_bAlive;
	}
	void Piece::Snapshot( void ) {

		m_iMovesSnapshot = m_iMoves;
		m_PosSnapshot = m_Pos;
		m_bAliveSnapshot = m_bAlive;

	}
	void Piece::Revert( void ) {

		m_iMoves = m_iMovesSnapshot;
		m_Pos = m_PosSnapshot;
		m_bAlive = m_bAliveSnapshot;

	}
	void Piece::RefreshMoves( bool bVerify ) {

		GenerateMoves( &m_Moves );
		if( bVerify ) {

			SpaceIterator iter = m_Moves.begin();
			while( iter != m_Moves.end() ) {

				if( !m_pBoard->VerifyMove( this, (*iter) ) ) iter = m_Moves.erase( iter );
				else ++iter;

			}

		}

	}
	/**
	 * Implementation for class Pawn
	 */
	Pawn::Pawn( Board *pBoard, Player owner, int x, int y ) : Piece( pBoard, owner, x, y ) {
		// do nothing
	}
	Pawn::~Pawn( void ) {
		// do nothing
	}
	void Pawn::GenerateMoves( SpaceList *pList ) {

		BoardSpace move;
		pList->clear();
		if( !Alive() ) return;

		// Create the Normal Move for a Pawn...
		move = Space( m_Pos.iX, m_Pos.iY + FORWARD[m_Owner], true, false, ( move.iY == FAR_ROW[m_Owner] ) ? SM_PROMOTE : SM_NONE );
		if( ValidSpace( move ) && m_pBoard->GetPiece( move ) == NULL ) {

			pList->push_back( move );

			// Create the 'First move only' move for a Pawn...
			if( m_iMoves == 0 ) {
				move = Space( m_Pos.iX, m_Pos.iY + FORWARD[m_Owner] * 2, true, false, SM_NONE );
				if( ValidSpace( move ) && m_pBoard->GetPiece( move ) == NULL ) pList->push_back( move );
			}

		}

		// Create the Moves on diagonals if there are 'capturable' targets there
		BoardSpace rcap = Space( m_Pos.iX + RIGHT[m_Owner], m_Pos.iY + FORWARD[m_Owner], true, true, SM_NONE );
		BoardSpace lcap = Space( m_Pos.iX + LEFT[m_Owner], m_Pos.iY + FORWARD[m_Owner], true, true, SM_NONE );

		LPPIECE lpiece = m_pBoard->GetPiece( lcap ), rpiece = m_pBoard->GetPiece( rcap );

		if( ValidSpace( rcap ) && rpiece != NULL && rpiece->Owner() != m_Owner ) pList->push_back( rcap );
		if( ValidSpace( lcap ) && lpiece != NULL && lpiece->Owner() != m_Owner ) pList->push_back( lcap );

		// Quick escape for En Passant
		if( m_Pos.iY != PAWN_ROW[m_Owner] + FORWARD[m_Owner] * 3 ) return;

		// Create the possible En Passant Moves
		BoardSpace ep_left = Space( m_Pos.iX + LEFT[m_Owner], m_Pos.iY + FORWARD[m_Owner], true, false, SM_EN_PASSANT );
		BoardSpace ep_right = Space( m_Pos.iX + RIGHT[m_Owner], m_Pos.iY + FORWARD[m_Owner], true, false, SM_EN_PASSANT );
		BoardSpace ep_lcap = Space( m_Pos.iX + LEFT[m_Owner], m_Pos.iY, true, false, SM_NONE );
		BoardSpace ep_rcap = Space( m_Pos.iX + RIGHT[m_Owner], m_Pos.iY, true, false, SM_NONE );

		rpiece = m_pBoard->GetPiece( ep_rcap ); lpiece = m_pBoard->GetPiece( ep_lcap );
		LPPIECE lenp = m_pBoard->GetPiece( ep_left ), renp = m_pBoard->GetPiece( ep_right );

		if( ValidSpace( ep_left ) && lenp == NULL && lpiece != NULL && lpiece->VulnerableToEnPassant() ) pList->push_back( ep_left );
		if( ValidSpace( ep_right ) && renp == NULL && rpiece != NULL && rpiece->VulnerableToEnPassant() ) pList->push_back( ep_right );

	}
	void Pawn::Draw( void ) {

		// sanity check
		if( !m_bAlive ) return;

		OpenGL *pGL = OpenGL::GetSingleton();

		// Set up our Matrix...
		pGL->Push();
		pGL->Identity();
		pGL->Translate( BoardStartX + SpaceSize * m_Pos.iX, BoardStartY + SpaceSize * m_Pos.iY, 0.0f );

		Color c = ( (m_Owner == BLACK) ? Color( 0.0f, 0.0f, 0.0f ) : Color( 1.0f, 1.0f, 1.0f ) );
		c.Activate();

		pGL->Begin( GL_QUADS );

		// Head
		pGL->Vertex( -1.0f, -1.0f, -1 );
		pGL->Vertex(  1.0f, -1.0f, -1 );
		pGL->Vertex(  2.0f,  0.0f, -1 );
		pGL->Vertex( -2.0f,  0.0f, -1 );
		pGL->Vertex( -1.0f,  1.0f, -1 );
		pGL->Vertex(  1.0f,  1.0f, -1 );
		pGL->Vertex(  2.0f,  0.0f, -1 );
		pGL->Vertex( -2.0f,  0.0f, -1 );

		// Neck
		pGL->Vertex( -1.0f,  1.0f, -1 );
		pGL->Vertex(  1.0f,  1.0f, -1 );
		pGL->Vertex(  1.5f,  2.0f, -1 );
		pGL->Vertex( -1.5f,  2.0f, -1 );

		// Base
		pGL->Vertex( -3.0f,  4.0f, -1 );
		pGL->Vertex( -3.0f,  2.0f, -1 );
		pGL->Vertex(  3.0f,  2.0f, -1 );
		pGL->Vertex(  3.0f,  4.0f, -1 );

		pGL->End();
		pGL->Flush();

		// Restore the Matrix...
		pGL->Pop();

	}
	bool Pawn::CanBePromoted( void ) {
		return ( m_Pos.iY == FAR_ROW[m_Owner] );
	}
	bool Pawn::VulnerableToCheck( void ) {
		return false;
	}
	bool Pawn::VulnerableToEnPassant( void ) {
		return ( m_iMoves == 1 && m_Pos.iY == PAWN_ROW[m_Owner] + FORWARD[m_Owner] * 2 );
	}
	bool Pawn::CanCastle( void ) {
		return false;
	}
	bool Pawn::CanBeCastled( void ) {
		return false;
	}
	/**
	 * Implementation for class Rook
	 */
	Rook::Rook( Board *pBoard, Player owner, int x, int y ) : Piece( pBoard, owner, x, y ) {
		// do nothing
	}
	Rook::~Rook( void ) {
		// do nothing
	}
	void Rook::GenerateMoves( SpaceList *pList ) {

		int dirs[4][2] = {	{ 0, FORWARD[m_Owner] },
							{ RIGHT[m_Owner], 0 },
							{ 0, BACKWARD[m_Owner] },
							{ LEFT[m_Owner], 0 } };

		pList->clear();
		if( !Alive() ) return;

		// Generate moves...
		for( int i = 0; i < 4; ++i ) {

			int d = 1;
			while( true ) {

				BoardSpace move = Space( m_Pos.iX + dirs[i][0] * d, m_Pos.iY + dirs[i][1] * d, true, true, SM_NONE );
				if( !ValidSpace( move ) ) break;
				LPPIECE pPiece = m_pBoard->GetPiece( move );
				if( pPiece && pPiece->Alive() ) {
					if( pPiece->Owner() == m_Owner ) move.bLegal = false;
					pList->push_back( move );
					break;
				}
				pList->push_back( move );
				++d;

			}

		}

	}
	void Rook::Draw( void ) {

		// sanity check
		if( !m_bAlive ) return;

		OpenGL *pGL = OpenGL::GetSingleton();

		// Set up our Matrix...
		pGL->Push();
		pGL->Identity();
		pGL->Translate( BoardStartX + SpaceSize * m_Pos.iX, BoardStartY + SpaceSize * m_Pos.iY, 0.0f );

		Color c = ( (m_Owner == BLACK) ? Color( 0.0f, 0.0f, 0.0f ) : Color( 1.0f, 1.0f, 1.0f ) );
		c.Activate();

		pGL->Begin( GL_QUADS );

		// Battlements
		pGL->Vertex( -3.0f, -3.0f, -1 );
		pGL->Vertex( -3.0f, -4.0f, -1 );
		pGL->Vertex( -2.0f, -4.0f, -1 );
		pGL->Vertex( -2.0f, -3.0f, -1 );
		pGL->Vertex(  3.0f, -3.0f, -1 );
		pGL->Vertex(  3.0f, -4.0f, -1 );
		pGL->Vertex(  2.0f, -4.0f, -1 );
		pGL->Vertex(  2.0f, -3.0f, -1 );
		pGL->Vertex( -1.0f, -3.0f, -1 );
		pGL->Vertex( -1.0f, -4.0f, -1 );
		pGL->Vertex(  1.0f, -4.0f, -1 );
		pGL->Vertex(  1.0f, -3.0f, -1 );

		// Head
		pGL->Vertex( -3.0f, -3.0f, -1 );
		pGL->Vertex( -3.0f, -1.0f, -1 );
		pGL->Vertex(  3.0f, -1.0f, -1 );
		pGL->Vertex(  3.0f, -3.0f, -1 );

		// Neck
		pGL->Vertex( -2.0f,  2.0f, -1 );
		pGL->Vertex( -2.0f, -1.0f, -1 );
		pGL->Vertex(  2.0f, -1.0f, -1 );
		pGL->Vertex(  2.0f,  2.0f, -1 );

		// Base
		pGL->Vertex( -3.0f,  4.0f, -1 );
		pGL->Vertex( -3.0f,  2.0f, -1 );
		pGL->Vertex(  3.0f,  2.0f, -1 );
		pGL->Vertex(  3.0f,  4.0f, -1 );

		pGL->End();
		pGL->Flush();

		// Restore the Matrix...
		pGL->Pop();

	}
	bool Rook::CanBePromoted( void ) {
		return false;
	}
	bool Rook::CanBeCastled( void ) {

		if( m_iMoves > 0 ) return false;

		int iDir = (m_Pos.iX == 0) ? 1 : -1;

		LPPIECE pPiece = NULL;
		int i = 1;
		while( !pPiece ) {
			BoardSpace spot = { m_Pos.iX + iDir * i, m_Pos.iY, true, SM_NONE };
			pPiece = m_pBoard->GetPiece( spot );
			++i;
		}

		return pPiece->CanCastle();

	}
	/**
	 * Implementation for class Bishop
	 */
	Bishop::Bishop( Board *pBoard, Player owner, int x, int y ) : Piece( pBoard, owner, x, y ) {
		// do nothing
	}
	Bishop::~Bishop( void ) {
		// do nothing
	}
	void Bishop::GenerateMoves( SpaceList *pList ) {

		int dirs[4][2] = {	{ RIGHT[m_Owner], FORWARD[m_Owner] },
							{ LEFT[m_Owner], FORWARD[m_Owner] },
							{ RIGHT[m_Owner], BACKWARD[m_Owner] },
							{ LEFT[m_Owner], BACKWARD[m_Owner] } };

		pList->clear();
		if( !Alive() ) return;

		// Generate moves...
		for( int i = 0; i < 4; ++i ) {

			int d = 1;
			while( true ) {

				BoardSpace move = Space( m_Pos.iX + dirs[i][0] * d, m_Pos.iY + dirs[i][1] * d, true, true, SM_NONE );
				if( !ValidSpace( move ) ) break;
				LPPIECE pPiece = m_pBoard->GetPiece( move );
				if( pPiece && pPiece->Alive() ) {
					if( pPiece->Owner() == m_Owner ) move.bLegal = false;
					pList->push_back( move );
					break;
				}
				pList->push_back( move );
				++d;

			}

		}

	}
	void Bishop::Draw( void ) {

		// sanity check
		if( !m_bAlive ) return;

		OpenGL *pGL = OpenGL::GetSingleton();

		// Set up our Matrix...
		pGL->Push();
		pGL->Identity();
		pGL->Translate( BoardStartX + SpaceSize * m_Pos.iX, BoardStartY + SpaceSize * m_Pos.iY, 0.0f );

		Color c = ( (m_Owner == BLACK) ? Color( 0.0f, 0.0f, 0.0f ) : Color( 1.0f, 1.0f, 1.0f ) );
		c.Activate();

		pGL->Begin( GL_QUADS );

		// Head
		pGL->Vertex(  0.0f, -4.0f, -1 );
		pGL->Vertex( -2.0f,  0.0f, -1 );
		pGL->Vertex(  0.0f,  4.0f, -1 );
		pGL->Vertex(  2.0f,  0.0f, -1 );

		// Base
		pGL->Vertex( -3.0f,  4.0f, -1 );
		pGL->Vertex( -3.0f,  2.0f, -1 );
		pGL->Vertex(  3.0f,  2.0f, -1 );
		pGL->Vertex(  3.0f,  4.0f, -1 );


		pGL->End();
		pGL->Flush();

		// Restore the Matrix...
		pGL->Pop();

	}
	bool Bishop::CanBePromoted( void ) {
		return false;
	}
	/**
	 * Implementation for class Knight
	 */
	Knight::Knight( Board *pBoard, Player owner, int x, int y ) : Piece( pBoard, owner, x, y ) {
		// do nothing
	}
	Knight::~Knight( void ) {
		// do nothing
	}
	void Knight::GenerateMoves( SpaceList *pList ) {

		int moves[8][2] = { { FORWARD[m_Owner] * 2, 	RIGHT[m_Owner] },
							{ FORWARD[m_Owner] * 2, 	LEFT[m_Owner] },
							{ BACKWARD[m_Owner] * 2, 	RIGHT[m_Owner] },
							{ BACKWARD[m_Owner] * 2, 	LEFT[m_Owner] },
							{ FORWARD[m_Owner],			RIGHT[m_Owner] * 2 },
							{ BACKWARD[m_Owner],		RIGHT[m_Owner] * 2 },
							{ FORWARD[m_Owner],			LEFT[m_Owner] * 2 },
							{ BACKWARD[m_Owner],		LEFT[m_Owner] * 2 } };

		pList->clear();
		if( !Alive() ) return;

		for( int i=0; i< 8; ++i ) {

			BoardSpace move = Space( m_Pos.iX + moves[i][0], m_Pos.iY + moves[i][1], true, true, SM_NONE );
			if( !ValidSpace( move ) ) continue;
			LPPIECE pOccupant = m_pBoard->GetPiece( move );
			if( pOccupant && pOccupant->Owner() == m_Owner ) continue;
			pList->push_back( move );

		}

	}
	void Knight::Draw( void ) {

		// sanity check
		if( !m_bAlive ) return;

		OpenGL *pGL = OpenGL::GetSingleton();

		// Set up our Matrix...
		pGL->Push();
		pGL->Identity();
		pGL->Translate( BoardStartX + SpaceSize * m_Pos.iX, BoardStartY + SpaceSize * m_Pos.iY, 0.0f );

		Color c = ( (m_Owner == BLACK) ? Color( 0.0f, 0.0f, 0.0f ) : Color( 1.0f, 1.0f, 1.0f ) );
		c.Activate();

		pGL->Begin( GL_QUADS );

		// Neck
		pGL->Vertex( -0.5f, -1.0f, -1 );
		pGL->Vertex( -2.0f,  2.0f, -1 );
		pGL->Vertex(  2.0f,  2.0f, -1 );
		pGL->Vertex(  2.5f, -1.0f, -1 );

		// Head
		pGL->Vertex(  0.0f, -4.0f, -1 );
		pGL->Vertex(  2.0f, -4.0f, -1 );
		pGL->Vertex(  2.5f, -1.0f, -1 );
		pGL->Vertex(  0.0f, -1.0f, -1 );

		// Mouth
		pGL->Vertex( -2.5f, -1.5f, -1 );
		pGL->Vertex(  0.0f, -4.0f, -1 );
		pGL->Vertex(  0.0f, -1.0f, -1 );
		pGL->Vertex( -2.5f,  0.0f, -1 );

		// Base
		pGL->Vertex( -3.0f,  4.0f, -1 );
		pGL->Vertex( -3.0f,  2.0f, -1 );
		pGL->Vertex(  3.0f,  2.0f, -1 );
		pGL->Vertex(  3.0f,  4.0f, -1 );

		pGL->End();
		pGL->Flush();

		// Restore the Matrix...
		pGL->Pop();

	}
	bool Knight::CanBePromoted( void ) {
		return false;
	}
	/**
	 * Implementation for class Queen
	 */
	Queen::Queen( Board *pBoard, Player owner, int x, int y ) : Piece( pBoard, owner, x, y ) {
		// do nothing
	}
	Queen::~Queen( void ) {
		// do nothing
	}
	void Queen::GenerateMoves( SpaceList *pList ) {

		int dirs[8][2] = {	{ RIGHT[m_Owner], FORWARD[m_Owner] },
							{ LEFT[m_Owner], FORWARD[m_Owner] },
							{ RIGHT[m_Owner], BACKWARD[m_Owner] },
							{ LEFT[m_Owner], BACKWARD[m_Owner] },
							{ 0, FORWARD[m_Owner] },
							{ RIGHT[m_Owner], 0 },
							{ 0, BACKWARD[m_Owner] },
							{ LEFT[m_Owner], 0 } };

		pList->clear();
		if( !Alive() ) return;

		// Generate moves...
		for( int i = 0; i < 8; ++i ) {

			int d = 1;
			while( true ) {

				BoardSpace move = Space( m_Pos.iX + dirs[i][0] * d, m_Pos.iY + dirs[i][1] * d, true, true, SM_NONE );
				if( !ValidSpace( move ) ) break;
				LPPIECE pPiece = m_pBoard->GetPiece( move );
				if( pPiece && pPiece->Alive() ) {
					if( pPiece->Owner() == m_Owner ) move.bLegal = false;
					pList->push_back( move );
					break;
				}
				pList->push_back( move );
				++d;

			}

		}

	}
	void Queen::Draw( void ) {

		// sanity check
		if( !m_bAlive ) return;

		OpenGL *pGL = OpenGL::GetSingleton();

		// Set up our Matrix...
		pGL->Push();
		pGL->Identity();
		pGL->Translate( BoardStartX + SpaceSize * m_Pos.iX, BoardStartY + SpaceSize * m_Pos.iY, 0.0f );

		Color c = ( (m_Owner == BLACK) ? Color( 0.0f, 0.0f, 0.0f ) : Color( 1.0f, 1.0f, 1.0f ) );
		c.Activate();

		pGL->Begin( GL_QUADS );

		// Center Point
		pGL->Vertex(  0.0f, -4.0f, -1 );
		pGL->Vertex( -1.5f,  0.0f, -1 );
		pGL->Vertex(  0.0f,  4.0f, -1 );
		pGL->Vertex(  1.5f,  0.0f, -1 );

		// Middle Points
		pGL->Vertex( -3.0f, -2.0f, -1 );
		pGL->Vertex(  0.0f, -1.0f, -1 );
		pGL->Vertex(  0.0f,  0.0f, -1 );
		pGL->Vertex( -2.0f,  0.0f, -1 );
		pGL->Vertex(  3.0f, -2.0f, -1 );
		pGL->Vertex(  0.0f, -1.0f, -1 );
		pGL->Vertex(  0.0f,  0.0f, -1 );
		pGL->Vertex(  2.0f,  0.0f, -1 );

		// Outer Points
		pGL->Vertex( -4.0f,  0.0f, -1 );
		pGL->Vertex( -2.0f,  2.0f, -1 );
		pGL->Vertex(  2.0f,  2.0f, -1 );
		pGL->Vertex(  4.0f,  0.0f, -1 );

		// Base
		pGL->Vertex( -3.0f,  4.0f, -1 );
		pGL->Vertex( -3.0f,  2.0f, -1 );
		pGL->Vertex(  3.0f,  2.0f, -1 );
		pGL->Vertex(  3.0f,  4.0f, -1 );

		pGL->End();
		pGL->Flush();

		// Restore the Matrix...
		pGL->Pop();

	}
	bool Queen::CanBePromoted( void ) {
		return false;
	}
	/**
	 * Implementation for class King
	 */
	King::King( Board *pBoard, Player owner, int x, int y ) : Piece( pBoard, owner, x, y ) {
		// do nothing
	}
	King::~King( void ) {
		// do nothing
	}
	void King::GenerateMoves( SpaceList *pList ) {

		int moves[8][2] = { { FORWARD[m_Owner], 0 },
							{ FORWARD[m_Owner], RIGHT[m_Owner] },
							{ FORWARD[m_Owner], LEFT[m_Owner] },
							{ BACKWARD[m_Owner], 0 },
							{ BACKWARD[m_Owner], RIGHT[m_Owner] },
							{ BACKWARD[m_Owner], LEFT[m_Owner] },
							{ 0, RIGHT[m_Owner] },
							{ 0, LEFT[m_Owner] } };

		pList->clear();
		if( !Alive() ) return;

		for( int i=0; i< 8; ++i ) {

			BoardSpace move = Space( m_Pos.iX + moves[i][0], m_Pos.iY + moves[i][1], true, true, SM_NONE );
			if( !ValidSpace( move ) ) continue;
			LPPIECE pOccupant = m_pBoard->GetPiece( move );
			if( pOccupant && pOccupant->Owner() == m_Owner ) continue;
			if( m_pBoard->Threatened( move, m_Owner == WHITE ? BLACK : WHITE ) ) continue;
			pList->push_back( move );

		}

		if( m_iMoves == 0 ) {
			Castle( RIGHT[m_Owner], pList );
			Castle( LEFT[m_Owner], pList );
		}

	}
	void King::Draw( void ) {

		// sanity check
		if( !m_bAlive ) return;

		OpenGL *pGL = OpenGL::GetSingleton();

		// Set up our Matrix...
		pGL->Push();
		pGL->Identity();
		pGL->Translate( BoardStartX + SpaceSize * m_Pos.iX, BoardStartY + SpaceSize * m_Pos.iY, 0.0f );

		Color c = ( (m_Owner == BLACK) ? Color( 0.0f, 0.0f, 0.0f ) : Color( 1.0f, 1.0f, 1.0f ) );
		c.Activate();

		pGL->Begin( GL_QUADS );

		// Cross
		pGL->Vertex( -0.5f, -4.0f, -1 );
		pGL->Vertex(  0.5f, -4.0f, -1 );
		pGL->Vertex(  0.5f, -1.0f, -1 );
		pGL->Vertex( -0.5f, -1.0f, -1 );
		pGL->Vertex( -1.5f, -3.0f, -1 );
		pGL->Vertex(  1.5f, -3.0f, -1 );
		pGL->Vertex(  1.5f, -2.0f, -1 );
		pGL->Vertex( -1.5f, -2.0f, -1 );

		// Head
		pGL->Vertex( -2.0f,  2.0f, -1 );
		pGL->Vertex( -2.0f,  -1.0f, -1 );
		pGL->Vertex(  2.0f,  -1.0f, -1 );
		pGL->Vertex(  2.0f,  2.0f, -1 );

		// Base
		pGL->Vertex( -3.0f,  4.0f, -1 );
		pGL->Vertex( -3.0f,  2.0f, -1 );
		pGL->Vertex(  3.0f,  2.0f, -1 );
		pGL->Vertex(  3.0f,  4.0f, -1 );

		pGL->End();
		pGL->Flush();

		// Restore the Matrix...
		pGL->Pop();

	}
	bool King::CanBePromoted( void ) {
		return false;
	}
	void King::Castle( int iDir, SpaceList *pList ) {

		// left is long, right is short

		BoardSpace castle = Space( (iDir == 1) ? 7 : 0, HOME_ROW[m_Owner], true, false, SM_NONE );
		LPPIECE pCastle = m_pBoard->GetPiece( castle );
		if( !pCastle || !(pCastle->CanBeCastled()) ) return;

		// check for threats...
		for( int i = 0; i< 2; ++i ) {

			BoardSpace threat = Space( m_Pos.iX - iDir * i, m_Pos.iY, true, true, SM_NONE );
			if( m_pBoard->Threatened( threat, (m_Owner == WHITE) ? BLACK : WHITE ) ) return;

		}

		BoardSpace move = Space( m_Pos.iX + iDir * 2, m_Pos.iY, true, false, (castle.iX == 7) ? SM_SHORT_CASTLE : SM_LONG_CASTLE );
		pList->push_back( move );

	}

}// namespace Game
