/** \file piece.h
 * Defines class Piece and its Subclasses
 * \author Robert W Martin
 * \date 03-19-2007
 */

#ifndef _PIECE_H_
#define _PIECE_H_

/**
 * Include Files
 */
#include "interface.h"
#include "root.h"

/**
 * Namespace Game
 */
namespace Game {

	/** \class Piece
	 * Represents a Generic Piece Interface
	 */
	class Piece {

		friend class Board;
	protected:
		Board			*m_pBoard;
		BoardSpace		m_Pos;
		Player			m_Owner;
		bool			m_bAlive;
		SpaceList		m_Moves;
		int				m_iMoves;

		BoardSpace		m_PosSnapshot;
		bool			m_bAliveSnapshot;
		int				m_iMovesSnapshot;

		virtual void GenerateMoves( SpaceList *pList ) = 0;
	public:
		Piece( Board *pBoard, Player owner, int x, int y );
		virtual ~Piece( void );
		virtual void Draw( void ) = 0;
		SpaceList GetMoves( void );
		bool Move( BoardSpace move );
		void Capture( void );
		Player Owner( void );
		BoardSpace GetPosition( void );
		bool Alive( void );
		virtual bool CanBePromoted( void ) = 0;
		virtual bool VulnerableToCheck( void ) = 0;
		virtual bool VulnerableToEnPassant( void ) = 0;
		virtual bool CanCastle( void ) = 0;
		virtual bool CanBeCastled( void ) = 0;
		void Snapshot( void );
		void Revert( void );
		void RefreshMoves( bool bVerify = true );

	}; // class Piece

	/** \class Pawn
	 * Represents a Pawn
	 */
	class Pawn : public Piece {

		friend class Board;
	protected:

		virtual void GenerateMoves( SpaceList *pList );
	public:
		Pawn( Board *pBoard, Player owner, int x, int y );
		virtual ~Pawn( void );
		virtual void Draw( void );
		virtual bool CanBePromoted( void );
		virtual bool VulnerableToCheck( void );
		virtual bool VulnerableToEnPassant( void );
		virtual bool CanCastle( void );
		virtual bool CanBeCastled( void );

	};// class Pawn

	/** \class Rook
	 * Represents a Rook
	 */
	class Rook : public Piece {

		friend class Board;
	protected:

		virtual void GenerateMoves( SpaceList *pList );
	public:
		Rook( Board *pBoard, Player owner, int x, int y );
		virtual ~Rook( void );
		virtual void Draw( void );
		virtual bool CanBePromoted( void );
		virtual bool VulnerableToCheck( void ) { return false; }
		virtual bool VulnerableToEnPassant( void ) { return false; }
		virtual bool CanCastle( void ) { return false; }
		virtual bool CanBeCastled( void );

	};// class Rook

	/** \class Bishop
	 * Represents a Bishop
	 */
	class Bishop : public Piece {

		friend class Board;
	protected:

		virtual void GenerateMoves( SpaceList *pList );
	public:
		Bishop( Board *pBoard, Player owner, int x, int y );
		virtual ~Bishop( void );
		virtual void Draw( void );
		virtual bool CanBePromoted( void );
		virtual bool VulnerableToCheck( void ) { return false; }
		virtual bool VulnerableToEnPassant( void ) { return false; }
		virtual bool CanCastle( void ) { return false; }
		virtual bool CanBeCastled( void ) { return false; }

	};// class Bishop


	/** \class Knight
	 * Represents a Knight
	 */
	class Knight : public Piece {

		friend class Board;
	protected:

		virtual void GenerateMoves( SpaceList *pList );
	public:
		Knight( Board *pBoard, Player owner, int x, int y );
		virtual ~Knight( void );
		virtual void Draw( void );
		virtual bool CanBePromoted( void );
		virtual bool VulnerableToCheck( void ) { return false; }
		virtual bool VulnerableToEnPassant( void ) { return false; }
		virtual bool CanCastle( void ) { return false; }
		virtual bool CanBeCastled( void ) { return false; }

	};// class Knight

	/** \class Queen
	 * Represents a Queen
	 */
	class Queen : public Piece {

		friend class Board;
	protected:

		virtual void GenerateMoves( SpaceList *pList );
	public:
		Queen( Board *pBoard, Player owner, int x, int y );
		virtual ~Queen( void );
		virtual void Draw( void );
		virtual bool CanBePromoted( void );
		virtual bool VulnerableToCheck( void ) { return false; }
		virtual bool VulnerableToEnPassant( void ) { return false; }
		virtual bool CanCastle( void ) { return false; }
		virtual bool CanBeCastled( void ) { return false; }

	};// class Queen

	/** \class King
	* Represents a King
	*/
	class King : public Piece {

		friend class Board;
	protected:

		virtual void GenerateMoves( SpaceList *pList );
		void Castle( int iDir, SpaceList *pList );
	public:
		King( Board *pBoard, Player owner, int x, int y );
		virtual ~King( void );
		virtual void Draw( void );
		virtual bool CanBePromoted( void );
		virtual bool VulnerableToCheck( void ) { return true; }
		virtual bool VulnerableToEnPassant( void ) { return false; }
		virtual bool CanCastle( void ) { return true; }
		virtual bool CanBeCastled( void ) { return false; }

	};// class King

}// namespace Game

#endif
