/** \file root.h
 * Root Header File for the Game Module of RWM-Chess
 * \author Robert W Martin
 * \date 03-19-2007
 */

#ifndef _ROOT_H_
#define _ROOT_H_

/**
 * Include Files
 */
#include <vector>

namespace Game {

	/**
	 * Class Prototypes
	 */
	class Board;
	class Piece;
	typedef Piece *LPPIECE;

	/**
	 * This represents a Board Space
	 */
	typedef struct _BOARD_SPACE_ {
		int iX;
		int iY;
		bool bLegal;
		bool bThreat;
		int iSpecial;
	} BoardSpace;

	/**
	 * Create a BoardSpace
	 */
	inline BoardSpace Space( int iX, int iY, bool bLegal, bool bThreat, int iSpecial ) {
		BoardSpace bs;
		bs.iX = iX;
		bs.iY = iY;
		bs.bLegal = bLegal;
		bs.bThreat = bThreat;
		bs.iSpecial = iSpecial;
		return bs;
	}

	/**
	 * Define a List of Pieces
	 */
	typedef std::vector<LPPIECE>	PieceList;
	typedef PieceList::iterator	PieceIterator;

	/**
	 * Define a List of Board Spaces
	 */
	typedef std::vector<BoardSpace>	SpaceList;
	typedef SpaceList::iterator	SpaceIterator;

	/**
	 * Define the Players in the Game
	 */
	typedef unsigned int Player;
	const Player	BLACK = 1;
	const Player	WHITE = 0;
	const Player	NUM_PLAYERS = 2;
	const Player	OPPOSITE[2] = { BLACK, WHITE };

	/**
	 * Define the Status of the Game
	 */
	typedef unsigned int State;
	const State		GS_NORMAL = 0;
	const State		GS_CHECK[2] = { 1, 2 };
	const State		GS_CMATE[2]	= { 3, 4 };

	/**
	 * Special Moves
	 */
	const int 		SM_NONE = 0;
	const int		SM_SHORT_CASTLE = 1;
	const int		SM_LONG_CASTLE = 2;
	const int		SM_EN_PASSANT = 3;
	const int		SM_PROMOTE = 4;

	/**
	 * Rendering Constants
	 */
	const float		Border		=   2.5f;
	const float		BoardSize	=  40.0f;
	const float		BoardStartX	= -60.0f;
	const float		BoardStartY	= -35.0f;
	const float		SpaceSize	=  10.0f;
	const int		Spaces		=  8;

	/**
	 * Screen Size Constants
	 */
	const int		InitialW	=  900;
	const int		InitialH	=  600;
	const float		ScreenL		= -75.0f;
	const float		ScreenR		=  75.0f;
	const float		ScreenT		= -50.0f;
	const float		ScreenB		=  50.0f;

	/**
	 * Board Position & Movement Constants
	 */
	const int		HOME_ROW[2] = { 7, 0 };
	const int		PAWN_ROW[2] = { 6, 1 };
	const int		FAR_ROW[2]  = { 0, 7 };
	const int		FORWARD[2]  = { -1, 1 };
	const int		BACKWARD[2] = { 1, -1 };
	const int		RIGHT[2]    = { 1, -1 };
	const int		LEFT[2]		= { -1, 1 };

	/**
	 * Validate a BoardSpace
	 */
	inline bool ValidSpace( BoardSpace bs ) {
		return ( bs.iX >= 0 && bs.iX < Spaces && bs.iY >= 0 && bs.iY < Spaces );
	}

}

#endif

