/** \file interface.h
 * Defines functions and classes for interfacing with OpenGL and GLUT
 * \author Robert W Martin
 * \date 03-16-2007
 */

#ifndef _INTERFACE_H_
#define _INTERFACE_H_

/**
 * Include Files
 */
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

/** namespace Interface
 * Namespace that Contains all of the "Stuff" for OpenGL and GLUT
 */
namespace Interface {

	/**
	 * Callback Function Types
	 */
	typedef void (*DrawFunc)(void);
	typedef void (*ResizeFunc)(int w, int h);
	typedef void (*KboardFunc)(unsigned char c, int x, int y);
	typedef void (*MouseFunc)(int button, int state, int x, int y);
	typedef void (*MotionFunc)(int x, int y);

	/** \struct iRect
	 * Stores a Rectangle made of Integers
	 */
	typedef struct _I_RECT_ {
		int l;
		int r;
		int t;
		int b;
	} iRect;

	/** \class OpenGL
	 * Singleton class that manages the OpenGL Interface
	 * Uses OpenGL, and GLUT
	 */
	class OpenGL {

	private:
		static OpenGL			*m_pSingleton;
		int						m_iWindow;
		int						m_iWidth;
		int						m_iHeight;

		OpenGL( void );
		~OpenGL( void );
	public:
		/** \fn OpenGL *GetSingleton( void )
		 * Gets a Handle to the Singleton OpenGL Interface
		 * \return Pointer to OpenGL Singleton
		 */
		static OpenGL *GetSingleton( void );
		/** \fn void KillSingleton( void )
		 * Deallocates the Singleton OpenGL Interface
		 * \return void
		 */
		static void KillSingleton( void );
		/** \fn void Initialize( int argc, char **argv, iRect &window, char *pTitle )
		 * Initializes the OpenGL Rendering System
		 * \param argc The argc from int main(int argc, char **argv)
		 * \param argv The argv from int main(int argc, char **argv)
		 * \param iWidth The width of the window, in pixels
		 * \param iHeight The height of the window, in pixels
		 * \param pTitle Title of the Window
		 * \return void
		 */
		void Initialize( int argc, char **argv, int iWidth, int iHeight, char *pTitle );
		/** \fn void Shutdown( void )
		 * Shuts down the GLUT-powered window
		 * \return void
		 */
		void Shutdown( void );
		/** \fn void SetRenderer( DrawFunc draw, ResizeFunc resize )
		 * Sets the Rendering Function
		 * \param draw The rendering function to use
		 * \param resize The resizing function to use
		 * \return void
		 */
		void SetRenderer( DrawFunc draw, ResizeFunc resize );
		/** \fn void SetInput( KboardFunc kboard, MouseFunc mouse )
		 * Sets the Input Function callbacks
		 * \param kboard The callback for keyboard events
		 * \param mouse The callback for mouse events
		 * \return void
		 */
		void SetInput( KboardFunc kboard, MouseFunc mouse );
		/** \fn void StartMainLoop( void )
		 * Starts up the GLUT Main Loop
		 * \return void
		 */
		void StartMainLoop( void );
		/** \fn void Viewport( int iW, int iH )
		 * Sets the rendering viewport
		 * \param iW Width of the viewport
		 * \param iH Height of the viewport
		 * \return void
		 */
		void Viewport( int iW, int iH );
		/** \fn void PostRedisplay( void )
		 * Posts a Redisplay Event
		 * \return void
		 */
		void PostRedisplay( void );
		/** \fn void SwapBuffers( void )
		 * Swaps the Front and Back Buffers for smooth animation
		 * \return void
		 */
		void SwapBuffers( void );
		/** \fn void Blend( bool bBlend )
		 * Toggle the use of Alpha Blending
		 * \param bBlend use true to enable blending, false to disable
		 * \return void
		 */
		void Blend( bool bBlend );
		/** \fn void Clear( bool bPixel, bool bDepth )
		 * Clears either the pixel or depth buffers
		 * \param bPixel Do we clear the pixel buffer?
		 * \param bDepth Do we clear the depth buffer?
		 * \return void
		 */
		void Clear( bool bPixel, bool bDepth );
		/** \fn void Begin( int iMode )
		 * Being rendering
		 * \param iMode which type of primitive are we rendering?
		 * \return void
		 */
		void Begin( int iMode );
		/** \fn void End( void )
		 * End the rendering
		 * \return void
		 */
		void End( void );
		/** \fn void Flush( void )
		 * Flush the Vertex Buffer
		 * \return void
		 */
		void Flush( void );
		/** \fn void Vertex( float x, float y, float z )
		 * Push a vertex to the Video adapter
		 * \param x X-coordinate
		 * \param y Y-coordinate
		 * \param z Z-coordinate
		 * \return void
		 */
		void Vertex( float x, float y, float z );
		/** \fn void String( char *pString, float x, float y )
		 * Renders a String of Text
		 * \param pString String of text to render
		 * \param x X Coordinate to render at
		 * \param y Y Coordinate to render at
		 * \return void
		 */
		void String( char *pString, float x, float y );
		/** \fn void Mode( int iMode )
		 * Sets the Matrix Mode: GL_PROJECTION or GL_MODELVIEW
		 * \param iMode The Mode that we're switching to
		 * \return void
		 */
		static void Mode( int iMode );
		/** \fn void Identity( void )
		 * Loads the Identity Matrix into the Matrix Stack
		 * \return void
		 */
		static void Identity( void );
		/** \fn void Ortho( float fLeft, float fRight, float fTop, float fBottom )
		 * Loads an Orthographic Projection Matrix into the Matrix Stack
		 * \param fLeft Leftmost side of ortho projection
		 * \param fRight Rightmost side of ortho projection
		 * \param fTop Topmost side of ortho projection
		 * \param fBottom Bottommost side of ortho projection
		 * \return void
		 */
		static void Ortho( float fLeft, float fRight, float fTop, float fBottom );
		/** \fn void Translate( float x, float y, float z )
		 * Loads a translation matrix into the Matrix Stack
		 * \param x X coordinate of the translation vector
		 * \param y Y coordinate of the translation vector
		 * \param z Z coordinate of the translation vector
		 * \return void
		 */
		static void Translate( float x, float y, float z );
		/** \fn void Push( void )
		 * Pushes the Current matrix onto the stack
		 * \return void
		 */
		static void Push( void );
		/** \fn void Pop( void )
		 * Pops the Stack into the Current matrix
		 * \return void
		 */
		static void Pop( void );
		/** \fn int Width( void )
		 * Get the width of the screen, in pixels
		 * \return width of the screen
		 */
		int Width( void );
		/** \fn int Height( void )
		 * Get the height of the screen, in pixels
		 * \return height of the screen
		 */
		int Height( void );

	};// class OpenGL

	/** \class Color
	 * Represents an RGBA color with 4 floats
	 */
	class Color {

	private:
		float				m_R;
		float				m_G;
		float				m_B;
		float				m_A;
	public:
		Color( float r = 0.0f, float g = 0.0f, float b = 0.0f, float a = 1.0f );
		~Color( void );
		/** \fn void Set( float r, float g, float b, float a = 1.0f )
		 * Sets this color to the desired float values
		 * \param r Red component of color
		 * \param g Green component of color
		 * \param b Blue component of color
		 * \param a Alpha component of color
		 * \return void
		 */
		void Set( float r, float g, float b, float a = 1.0f );
		/** \fn
		 * Activates this color so that OpenCL will use it
		 * \return void
		 */
		void Activate( void );

	};// class Color

}// namespace Interface

#endif
